<?php
/**
 * Custom functions that act independently of the theme templates
 */

/**
 * Theme Info API.
 */
class Design_Orbital_Theme_Info {

	/**
	 * Variables required for the theme info
	 */
	 protected $remote_api_url       = null;
	 protected $theme_transient_name = null;
	 protected $theme_name           = null;
	 protected $theme_version        = null;
	 protected $theme_text_domain    = null;
	 protected $theme_home_url       = null;

	/**
	 * Initialize the class.
	 */
	function __construct( $args = array() ) {

		/**
		 * Define the array of defaults
		 */
		$defaults = array (
			'remote_api_url' => 'https://designorbital.market',
		);

		/**
		 * Parse incoming $args into an array and merge it with $defaults
		 */
		$args = wp_parse_args( $args, $defaults );

		// Set Class Properties
		$this->remote_api_url = $args['remote_api_url'];

		// Theme Info
		add_action( 'admin_init', array( $this, 'theme_info' ) );

		// Filter `http_request_args`
		add_filter( 'http_request_args', array( $this, 'disable_wporg_request' ), 5, 2 );

	}

	/**
	 * Theme Info
	 */
	function theme_info() {

		// Declare Variables
		$theme_transient_name = 'theme_info_';
		$theme_name           = 'Design Orbital';
		$theme_version        = '1.0';
		$theme_text_domain    = 'design-orbital';
		$theme_home_url       = 'https://designorbital.market';

		/**
		 * Gets a WP_Theme object for a theme.
		 *
		 * @see https://developer.wordpress.org/reference/functions/wp_get_theme/
		 */
		$theme_info = wp_get_theme();
		if ( is_object( $theme_info ) ) {
			$theme_name        = $theme_info->get( 'Name' );
			$theme_version     = $theme_info->get( 'Version' );
			$theme_text_domain = $theme_info->get( 'TextDomain' );
		}

		// Transient Name
		$theme_transient_name = $theme_transient_name . str_replace( '-', '_', $theme_text_domain );
		$theme_transient_name = $theme_transient_name . '_' . str_replace( '.', '_', $theme_version );

		// Set Class Properties
		$this->theme_transient_name = $theme_transient_name;
		$this->theme_name           = $theme_name;
		$this->theme_version        = $theme_version;
		$this->theme_text_domain    = $theme_text_domain;
		$this->theme_home_url       = home_url();

		// API Call
		$theme_info_transient = get_transient( $this->theme_transient_name );
		if ( false === $theme_info_transient ) {
			$response = $this->get_api_response();
		}
	}

	/**
	 * Makes a call to the API.
	 *
	 * @param array $api_params to be used for wp_remote_get.
	 * @return array $response decoded JSON response.
	 */
	function get_api_response() {

		// API Params
		$api_params = array(
			'designorbital_action' => 'theme_info',
			'theme_name'           => urlencode( $this->theme_name ),
			'theme_version'        => $this->theme_version ,
			'theme_text_domain'    => urlencode( $this->theme_text_domain ),
			'theme_home_url'       => $this->theme_home_url,
		);

		// Response Handling
		$response_flag    = false;
		$response_message = 'WP Remote Post Start';

		// Call the custom API.
		$response = wp_remote_post( $this->remote_api_url, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

		// Make sure the response came back okay.
		if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {

			// Error Rerporting
			if ( is_object( $response ) && is_wp_error( $response ) ) {
				$response_message = 'Remote Connection Error: ' . $response->get_error_message();
			} else {
				$response_message = 'Remote Connection Error: Please Try Again.';
			}

		} else {

			// Everything (Connection) is Okay, Now retrieve the remote body.
			$theme_info_remote_body = json_decode( wp_remote_retrieve_body( $response ) );

			// Validation
			if ( is_object( $theme_info_remote_body ) ) {

				// Message
				$response_flag = true;
				$response_message = 'Response Okay: ' . $theme_info_remote_body->message;


			} else {

				$response_message = 'Remote Body Error: Please Try Again.';

			}

		}

		// Transient Handling
		// Expiration 1 Day = 86400
		if ( true === $response_flag ) {
			set_transient( $this->theme_transient_name, $response_message, 2592000 );
		} else {
			set_transient( $this->theme_transient_name, $response_message, 86400 );
		}

		return $response;
	}

	/**
	 * Disable requests to wp.org repository for this theme.
	 */
	function disable_wporg_request( $r, $url ) {

		// If it's not a theme update request, bail.
		if ( 0 !== strpos( $url, 'https://api.wordpress.org/themes/update-check/1.1/' ) ) {
 			return $r;
 		}

 		// Decode the JSON response
 		$themes = json_decode( $r['body']['themes'] );

 		// Remove the active parent and child themes from the check
 		$parent = get_option( 'template' );
 		$child  = get_option( 'stylesheet' );
 		unset( $themes->themes->$parent );
 		unset( $themes->themes->$child );

 		// Encode the updated JSON response
 		$r['body']['themes'] = json_encode( $themes );

 		return $r;
	}

}

// For Admin Only
if ( is_admin() ) {

	/**
	 * Use WordPress Hook `after_setup_theme`
	 *
	 * @see https://codex.wordpress.org/Plugin_API/Action_Reference
	 */
	function design_orbital_load_theme_info_class() {

		// Class Instance
		$design_orbital_theme_info = new Design_Orbital_Theme_Info();

	}
	add_action( 'after_setup_theme', 'design_orbital_load_theme_info_class' );
}
